# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31 aw>

import unittest
from memacs.example import Foo


class TestFoo(unittest.TestCase):

    def setUp(self):
        pass

    def test_all(self):
        argv = "-s"
        memacs = Foo(argv=argv.split())
        # or when in append mode:
        # memacs = Foo(argv=argv.split(), append=True)
        data = memacs.test_get_entries()

        # generate assertEquals :)
        #for d in range(len(data)):
        #   print "self.assertEqual(\n\tdata[%d],\n\t\"%s\")" % \
        #        (d, data[d])

        self.assertEqual(
            data[0],
            "** <1970-01-01 Thu 00:00> foo")
        self.assertEqual(
            data[1],
            "   :PROPERTIES:")
        self.assertEqual(
            data[2],
            "   :ID:         e7663db158b7ba301fb23e3dc40347970c7f8a0f")
        self.assertEqual(
            data[3],
            "   :END:")
        self.assertEqual(
            data[4],
            "** <1970-01-01 Thu 00:00> bar\t:tag1:tag2:")
        self.assertEqual(
            data[5],
            "   bar notes")
        self.assertEqual(
            data[6],
            "   foo notes")
        self.assertEqual(
            data[7],
            "   :PROPERTIES:")
        self.assertEqual(
            data[8],
            "   :DESCRIPTION:  foooo")
        self.assertEqual(
            data[9],
            "   :FOO-PROPERTY: asdf")
        self.assertEqual(
            data[10],
            "   :ID:           97521347348df02dab8bf86fbb6817c0af333a3f")
        self.assertEqual(
            data[11],
            "   :END:")

    def tearDown(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31 aw>

from memacs.phonecalls import PhonecallsMemacs

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2012-03-08"
PROG_SHORT_DESCRIPTION = u"Memacs for phonecalls"
PROG_TAG = u"phonecalls"
PROG_DESCRIPTION = u"""
This Memacs module will parse output of phonecalls xml backup files

sample xml file:
<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>
<calls count="8">
  <call number="+43691234123" duration="59" date="1312563906092" type="1" />
  <call number="06612341234" duration="22" date="1312541215834" type="2" />
  <call number="-1" duration="382" date="1312530691081" type="1" />
  <call number="+4312341234" duration="289" date="1312482327195" type="1" />
  <call number="+4366412341234" duration="70" date="1312476334059" type="1" />
  <call number="+4366234123" duration="0" date="1312473751975" type="2" />
  <call number="+436612341234" duration="0" date="1312471300072" type="3" />
  <call number="+433123412" duration="60" date="1312468562489" type="2" />
</calls>

Then an Org-mode file is generated.
"""
COPYRIGHT_YEAR = "2011-2012"
COPYRIGHT_AUTHORS = """Karl Voit <tools@Karl-Voit.at>,
Armin Wieser <armin.wieser@gmail.com>"""

if __name__ == "__main__":
    memacs = PhonecallsMemacs(
        prog_version=PROG_VERSION_NUMBER,
        prog_version_date=PROG_VERSION_DATE,
        prog_description=PROG_DESCRIPTION,
        prog_short_description=PROG_SHORT_DESCRIPTION,
        prog_tag=PROG_TAG,
        copyright_year=COPYRIGHT_YEAR,
        copyright_authors=COPYRIGHT_AUTHORS
        )
    memacs.handle_main()



# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31 aw>

import unittest
import os
from memacs.rss import RssMemacs


class TestRss(unittest.TestCase):

    def setUp(self):
        self.test_file = os.path.dirname(
            os.path.abspath(__file__)) + os.sep + "tmp" \
            + os.path.sep + "sample-rss.txt"
        self.argv = "-s -f " + self.test_file

    def test_false_appending(self):
        try:
            memacs = RssMemacs(argv=self.argv.split())
            memacs.test_get_entries()
        except Exception:
            pass

    def test_all(self):
        memacs = RssMemacs(argv=self.argv.split())
        data = memacs.test_get_entries()

        # generate assertEquals :)
        #for d in range(len(data)):
        #    print "self.assertEqual(\n\tdata[%d],\n\t\"%s\")" % \
        #        (d, data[d])
        self.assertEqual(
            data[0],
            "** <2009-09-06 Sun 16:45> [[http://www.wikipedia.or" + \
            "g/][Example entry]]")
        self.assertEqual(
            data[1],
            "   Here is some text containing an interesting description.")
        self.assertEqual(
            data[2],
            "   :PROPERTIES:")
        self.assertEqual(
            data[3],
            "   :GUID:       unique string per item")
        self.assertEqual(
            data[4],
            "   :ID:         7ec7b2ec7d1ac5f18188352551b04f061af81e04")
        self.assertEqual(
            data[5],
            "   :END:")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Time-stamp: <2011-12-20 15:13:31 aw>

from memacs.git import GitMemacs

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2011-12-20"
PROG_SHORT_DESCRIPTION = u"Memacs for git files "
PROG_TAG = u"git"
PROG_DESCRIPTION = u"""
This class will parse files from git rev-parse output

use following command to generate input file
$ git rev-list --all --pretty=raw > /path/to/input file

Then an Org-mode file is generated that contains all commit message

If outputfile is specified, only non-existing commits are appended
"""
COPYRIGHT_YEAR = "2011-2012"
COPYRIGHT_AUTHORS = """Karl Voit <tools@Karl-Voit.at>,
Armin Wieser <armin.wieser@gmail.com>"""


if __name__ == "__main__":
    memacs = GitMemacs(
        prog_version=PROG_VERSION_NUMBER,
        prog_version_date=PROG_VERSION_DATE,
        prog_description=PROG_DESCRIPTION,
        prog_short_description=PROG_SHORT_DESCRIPTION,
        prog_tag=PROG_TAG,
        copyright_year=COPYRIGHT_YEAR,
        copyright_authors=COPYRIGHT_AUTHORS
        )
    memacs.handle_main()



# -*- coding: utf-8 -*-
# Time-stamp: <2012-04-16 23:04:44 armin>

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import os


class MemacsArgumentParser(ArgumentParser):
    """
    Inherited from Argumentparser

    MemacsArgumentParser handles default arguments which are needed for every
    Memacs module and gives a nicer output for help message.
    """

    def __init__(self,
                 prog_version,
                 prog_version_date,
                 prog_description,
                 copyright_year,
                 copyright_authors,
                 use_config_parser_name=""
                 ):

        self.__version = "%(prog)s v" + prog_version + " from " + \
            prog_version_date

        # format copyright authors:
        # indent from second author
        copyright_authors = copyright_authors.splitlines()
        for i in range(len(copyright_authors)):
            copyright_authors[i] = "            " + copyright_authors[i]
        copyright_authors = "\n".join(map(unicode, copyright_authors))

        epilog = ":copyright: (c) " + copyright_year + " by \n" + \
        copyright_authors + \
        "\n:license: GPL v2 or any later version\n" + \
        ":bugreports: https://github.com/novoid/Memacs\n" + \
        ":version: " + prog_version + " from " + prog_version_date + "\n"

        self.__use_config_parser_name = use_config_parser_name

        ArgumentParser.__init__(self,
                              description=prog_description,
                              add_help=True,
                              epilog=epilog,
                              formatter_class=RawDescriptionHelpFormatter
                              )
        self.__add_arguments()

    def __add_arguments(self):
        """
        Add's all standard arguments of a Memacs module
        """
        self.add_argument('--version',
                          action='version',
                          version=self.__version)

        self.add_argument("-v", "--verbose",
                          dest="verbose",
                          action="store_true",
                          help="enable verbose mode")

        self.add_argument("-s", "--suppress-messages",
                          dest="suppressmessages",
                          action="store_true",
                          help="do not show any log message " + \
                              "- helpful when -o not set")

        self.add_argument("-o", "--output",
                          dest="outputfile",
                          help="Org-mode file that will be generated " + \
                              " (see above). If no output file is given, " + \
                              "result gets printed to stdout",
                          metavar="FILE")

        self.add_argument("-a", "--append",
                          dest="append",
                          help="""when set and outputfile exists, then
                          only new entries are appendend.
                          criterion: :ID: property""",
                          action="store_true")

        self.add_argument("-t", "--tag",
                          dest="tag",
                          help="overriding tag: :Memacs:<tag>: (on top entry)")

        self.add_argument("--autotagfile",
                          dest="autotagfile",
                          help="file containing autotag information, see " + \
                          "doc file FAQs_and_Best_Practices.org",
                          metavar="FILE")
        # ---------------------
        # Config parser
        # ---------------------
        if self.__use_config_parser_name != "":
            self.add_argument("-c", "--config",
                              dest="configfile",
                              help="path to config file",
                              metavar="FILE")

    def parse_args(self, args=None, namespace=None):
        """
        overwriting ArgParser's parse_args and
        do checking default argument outputfile
        """
        args = ArgumentParser.parse_args(self, args=args, namespace=namespace)
        if args.outputfile:
            if not os.path.exists(os.path.dirname(args.outputfile)):
                self.error("Output file path(%s) does not exist!" %
                           args.outputfile)
            if not os.access(os.path.dirname(args.outputfile), os.W_OK):
                self.error("Output file %s is not writeable!" %
                           args.outputfile)
        else:
            if args.append:
                self.error("cannot set append when no outputfile specified")

        if args.suppressmessages == True and args.verbose == True:
            self.error("cannot set both verbose and suppress-messages")

        if args.autotagfile:
            if not os.path.exists(os.path.dirname(args.autotagfile)):
                self.error("Autotag file path(%s) doest not exist!" %
                           args.autotagfile)
            if not os.access(args.autotagfile, os.R_OK):
                self.error("Autotag file (%s) is not readable!" %
                           args.autotagfile)

        # ---------------------
        # Config parser
        # ---------------------
        if self.__use_config_parser_name != "":
            if args.configfile:
                if not os.path.exists(args.configfile):
                    self.error("Config file (%s) does not exist" %
                        args.configfile)
                if not os.access(args.configfile, os.R_OK):
                    self.error("Config file (%s) is not readable!" %
                        args.configfile)
            else:
                self.error("please specify a config file")
        return args



# -*- coding: utf-8 -*-
# Time-stamp: <2011-12-30 12:16:47 armin>

import unittest
from memacs.lib.mailparser import MailParser


class TestMailParser(unittest.TestCase):

    def test_parse_mail_without_body(self):
        message = """Date: Wed, 28 Dec 2011 14:02:00 +0100
From: Alice Ally <alice@ally.com>
To: Bob Bobby <Bob@bobby.com>
Subject: Bob sends a mesage
Message-ID: f2c1165a321d0e0@foo.com
X-Scanned-By: MIMEDefang 2.71 on 129.27.10.2

Hi!

Hope you can read my message

kind reagards,
Bob
        """
        timestamp, output, notes, properties = \
            MailParser.parse_message(message)

        self.assertEqual(timestamp, "<2011-12-28 Wed 14:02>")
        self.assertEqual(output, "[[mailto:alice@ally.com]" + \
                         "[Alice Ally]]: Bob sends a mesage")
        self.assertEqual(notes, "")
        p = """   :PROPERTIES:
   :TO:         Bob Bobby <Bob@bobby.com>
   :ID:         8fd560c32d51c455744df7abd26ea545924ba632
   :END:"""

        self.assertEqual(unicode(properties), p)

    def test_parse_mail_with_body(self):
        message = """Date: Wed, 28 Dec 2011 14:02:00 +0100
From: Alice Ally <alice@ally.com>
To: Bob Bobby <Bob@bobby.com>
Subject: Bob sends a mesage
Message-ID: f2c1165a321d0e0@foo.com
X-Scanned-By: MIMEDefang 2.71 on 129.27.10.2

Hi!

Hope you can read my message

kind reagards,
Bob"""
        timestamp, output, notes, properties = \
            MailParser.parse_message(message,
                                     True)

        self.assertEqual(timestamp, "<2011-12-28 Wed 14:02>")
        self.assertEqual(output, "[[mailto:alice@ally.com]" + \
                         "[Alice Ally]]: Bob sends a mesage")
        self.assertEqual(notes, "Hi!\n\nHope you can read my message\n" + \
                            "\nkind reagards,\nBob")
        p = """   :PROPERTIES:
   :TO:         Bob Bobby <Bob@bobby.com>
   :ID:         8fd560c32d51c455744df7abd26ea545924ba632
   :END:"""

        self.assertEqual(unicode(properties), p)

    def test_parse_ng_with_body(self):
        message = """Path: news.tugraz.at!not-for-mail
From: Alice Ally <alice@ally.com>
Newsgroups: tu-graz.betriebssysteme.linux
Subject: I love Memacs
Date: Thu, 17 Nov 2011 22:02:06 +0100
Message-ID: <2011-11-17T21-58-27@ally.com>
Reply-To: news@ally.com
Content-Type: text/plain; charset=utf-8

i just want to say that i love Memacs
"""
        timestamp, output, notes, properties = \
            MailParser.parse_message(message,
                                     True)

        self.assertEqual(timestamp, "<2011-11-17 Thu 22:02:06>")
        self.assertEqual(output,
                         "[[mailto:alice@ally.com][Alice Ally]]@[[news:tu-" + \
                         "graz.betriebssysteme.linux]" + \
                         "[tu-graz.betriebssysteme.linux]]: I love Memacs")
        self.assertEqual(notes, "i just want to say that i love Memacs\n")
        p = """   :PROPERTIES:\n   :REPLY-TO:   news@ally.com
   :NEWSGROUPS: tu-graz.betriebssysteme.linux
   :ID:         53e60f934645301478db6c9d5d3df71a043f9851
   :END:"""

        self.assertEqual(unicode(properties), p)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Time-stamp: <2012-03-09 15:28:53 armin>

import sys
import os
import logging
import xml.sax
import time
from xml.sax._exceptions import SAXParseException
from lib.orgformat import OrgFormat
from lib.memacs import Memacs
from lib.reader import CommonReader
from lib.orgproperty import OrgProperties


class PhonecallsSaxHandler(xml.sax.handler.ContentHandler):
    """
    Sax handler for following xml's:

    <?xml version='1.0' encoding='UTF-8' standalone='yes' ?>

    <calls count="8">
      <call number="+43691234123" duration="59" date="13193906092" type="1" />
      <call number="06612341234" duration="22" date="131254215834" type="2" />
      <call number="-1" duration="382" date="1312530691081" type="1" />
      <call number="+4312341234" duration="289" date="13124327195" type="1" />
      <call number="+4366412341234" duration="70" date="136334059" type="1" />
      <call number="+4366234123" duration="0" date="1312473751975" type="2" />
      <call number="+436612341234" duration="0" date="12471300072" type="3" />
      <call number="+433123412" duration="60" date="1312468562489" type="2" />
    </calls>"""

    def __init__(self,
                 writer,
                 ignore_incoming,
                 ignore_outgoing,
                 ignore_missed,
                 minimum_duration
                 ):
        """
        Ctor

        @param writer: orgwriter
        @param ignore_incoming: ignore incoming phonecalls
        @param ignore_outgoing: ignore outgoing phonecalls
        @param ignore_missed:   ignore missed   phonecalls
        @param minimum_duration:    ignore phonecalls less than that time
        """
        self._writer = writer
        self._ignore_incoming = ignore_incoming
        self._ignore_outgoing = ignore_outgoing
        self._ignore_missed = ignore_missed
        self._minimum_duration = minimum_duration

    def startElement(self, name, attrs):
        """
        at every <call> write to orgfile
        """
        logging.debug("Handler @startElement name=%s,attrs=%s", name, attrs)

        if name == "call":
            call_number = attrs['number']
            call_duration = int(attrs['duration'])
            call_date = int(attrs['date']) / 1000     # unix epoch

            call_type = int(attrs['type'])
            call_incoming = call_type == 1
            call_outgoing = call_type == 2
            call_missed = call_type == 3

            output = "Phonecall "

            skip = False

            if call_incoming:
                output += "from "
                if self._ignore_incoming:
                    skip = True
            elif call_outgoing:
                output += "to "
                if self._ignore_outgoing:
                    skip = True
            elif call_missed:
                output += "missed "
                if self._ignore_missed:
                    skip = True
            else:
                raise Exception("Invalid Phonecall Type: %d", call_type)

            if call_number != "-1":
                output += call_number
            else:
                output += "Unknown Number"

            output += " Duration: %d sec" % call_duration

            if call_duration < self._minimum_duration:
                skip = True

            timestamp = OrgFormat.datetime(time.gmtime(call_date))
            data_for_hashing = output + timestamp
            properties = OrgProperties(data_for_hashing=data_for_hashing)

            if not skip:
                self._writer.write_org_subitem(output=output,
                                               timestamp=timestamp,
                                               properties=properties
                                               )


class PhonecallsMemacs(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-f", "--file", dest="smsxmlfile",
            action="store", required=True,
            help="path to sms xml backup file")

        self._parser.add_argument(
            "--ignore-incoming", dest="ignore_incoming",
            action="store_true",
            help="ignore incoming phonecalls")

        self._parser.add_argument(
            "--ignore-outgoing", dest="ignore_outgoing",
            action="store_true",
            help="ignore outgoing phonecalls")

        self._parser.add_argument(
            "--ignore-missed", dest="ignore_missed",
            action="store_true",
            help="ignore outgoing phonecalls")

        self._parser.add_argument(
            "--minimum-duration", dest="minimum_duration",
            action="store", type=int,
            help="[sec] show only calls with duration >= this argument")

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)
        if not (os.path.exists(self._args.smsxmlfile) or \
                     os.access(self._args.smsxmlfile, os.R_OK)):
            self._parser.error("input file not found or not readable")

    def _main(self):
        """
        get's automatically called from Memacs class
        read the lines from phonecalls backup xml file,
        parse and write them to org file
        """

        data = CommonReader.get_data_from_file(self._args.smsxmlfile)

        try:
            xml.sax.parseString(data.encode('utf-8'),
                                PhonecallsSaxHandler(self._writer,
                                              self._args.ignore_incoming,
                                              self._args.ignore_outgoing,
                                              self._args.ignore_missed,
                                              self._args.minimum_duration,
                                              ))
        except SAXParseException:
            logging.error("No correct XML given")
            sys.exit(1)
